#!/bin/sh
###############################################################################
###############################################################################
#                               install
###############################################################################
###############################################################################
# copyright (C) 2002-2008 Andres Garcia Garcia - fandom@telefonica.net
# Distributed under the terms of the GPL v2
###############################################################################
###############################################################################
# Version 2.0
###############################################################################
###############################################################################
# This little script is only good for installing Getleft into your unix system
###############################################################################
# First of all we check whether we are going to use a console or graphic
# install
# \
if [ -n "$DISPLAY" ] ; then
# \
    if [ $# -eq 0 ] ; then
# \
        installSHELL="wish"
# \
    fi
# \
fi
# \
if [ -z $installSHELL ] ; then
# \
    installSHELL="tclsh"
# \
fi
# Now we have to check for a valid 'tclsh' or 'wish', we have to be so convoluted
# because FreeBSD, and I guess others, doesn't have a working shell in 'tclsh' or 
# 'wish', so you have to include the version number \
wish=`( for name in $(ls $(echo $PATH | sed 's/:/ /g') 2> /dev/null | grep ${installSHELL}8.5); do echo $name ; done ) | tail -1`
# \
[ -z "$wish" ] && wish=`(for name in $(ls $(echo $PATH | sed 's/:/ /g') 2> /dev/null | grep ${installSHELL}8.4); do echo $name ; done ) | tail -1`
# \
[ -z "$wish" ] && wish=${installSHELL}
# \
exec $wish "$0" ${1+"$@"}

catch {wm withdraw .}

################################################################################
# InstallDirs
#    Sets the directories in which we are going to install or uninstall the
#    program.
################################################################################
proc InstallDirs {} {
    global installDirs options

    if {$options(prefix)=="/usr"} {
        set installDirs(etc) /etc
    } else {
        set installDirs(etc) /usr/local/etc
    }
    set installDirs(bin) [file join $options(prefix) bin]
    set installDirs(lib) [file join $options(prefix) lib getleft]
    set installDirs(img) [file join $options(prefix) share getleft images]
    set installDirs(lan) [file join $options(prefix) share getleft languages]
    set docRoot [file join $options(prefix) share doc]
    if {[file exists $docRoot]} {
        set installDirs(doc) [file join $options(prefix) share doc getleft-1.2]
    } else {
        set installDirs(doc) [file join $options(prefix) doc getleft-1.2]
    }
    return
}

################################################################################
# WriteConfFile
#    Finally we write the file with the configuration.
################################################################################
proc WriteConfFile {} {
    global options consoleInstall installDirs

    set confFile [file join $installDirs(etc) getleft.conf]

    if {[catch {open $confFile w} handle]} {
        if {$consoleInstall==1} {
            puts "Couldn't open file: $handle"
        } else {
            Dialogos::Dialogo .error -type ok -icon error -title Error \
                -message "Couldn't open file: $handle"
        }
        exit
    }
    puts $handle "# Configure file for Getleft\n"
    puts $handle "\[scripts\]\n"
    puts $handle "$installDirs(lib)\n"
    puts $handle "\[doc\]\n"
    puts $handle "$installDirs(doc)\n"
    puts $handle "\[languages\]\n"
    puts $handle "$installDirs(lan)\n"
    puts $handle "\[images\]\n"
    puts $handle "$installDirs(img)\n"

    if {$options(proxy)!=""} {
        puts $handle "\[proxy\]\n"
        puts $handle "$options(proxy)\n"
        puts $handle "\[sameProxy\]\n"
        puts $handle "1\n"
    }
    puts $handle "\[language\]\n"
    puts $handle $options(lang)\n

    return
}

################################################################################
# InstallFiles
#    We get to do what we came here to do, get the files into your system.
################################################################################
proc InstallFiles {} {
    global options installDirs

    InstallDirs

    file copy -force Getleft.tcl $installDirs(bin)
    if {![file exists $installDirs(lib)]} {
        file mkdir $installDirs(lib)
    }
    foreach scriptFile [glob scripts/*] {
        file copy -force $scriptFile $installDirs(lib)
    }
    if {![file exists $installDirs(img)]} {
        file mkdir $installDirs(img)
    }
    foreach file [glob -nocomplain [file join $installDirs(img) *]] {
        file delete -force $file
    }
    foreach imgFile [glob images/*] {
        file copy -force $imgFile $installDirs(img)
    }
    if {![file exists $installDirs(lan)]} {
        file mkdir $installDirs(lan)
    }
    foreach lanFile [glob Languages/*] {
        file copy -force $lanFile $installDirs(lan)
    }
    if {![file exists $installDirs(doc)]} {
        file mkdir $installDirs(doc)
    }
    foreach docFile [glob doc/*] {
        file copy -force $docFile $installDirs(doc)
    }
    return
}

################################################################################
# ShowUsage
#    Shows a little message with the expected usage.
################################################################################
proc ShowUsage {} {

    puts "Usage: install.tcl ?--prefix=/path? ?--lang=lanCode? ?--personal=1|0?\n\
                 ?--proxy=proxyIp:proxyPort? ?--desktop=1|0? ?--menu=1|0?"

    exit
}

################################################################################
# ValidPath
#    Checks whether the given path is ok.
#
# Returns
#    '0': Ok
#    '1': the path isn't ok.
#    '2': the user can't write in the given directory.
################################################################################
proc ValidPath {path} {
    global env

    if {($path ne "/usr") && ($path ne "/usr/local")} {
        return 1
    }
    if {![file writable $path]} {
        return 2
    }
    return 0
}

################################################################################
# ValidLang
#    Checks whether the code represents a supported language.
#
# Returns
#    '1': the language is supported.
#    '0': it isn't
################################################################################
proc ValidLang {code} {

    if {($code=="en")||($code=="es")||($code=="fr")||($code=="pl")        \
            ||($code=="ko")||($code=="de")||($code=="nl")||($code=="it")  \
            ||($code=="pt")||($code=="eo")||($code=="ru")||($code=="zht") \
            ||($code=="tr")} {
        return 1
    }
    return 0
}

################################################################################
# ValidProxy
#    Checks whether the given proxy looks right.
#
# Returns
#    '1': it looks ok.
#    '0': it doesn't
################################################################################
proc ValidProxy {proxy} {
    return [regexp {(.+)(:)([0-9]+)} $proxy]
}

################################################################################
# ProcessArgs
#    Parse the command line for options.
################################################################################
proc ProcessArgs {} {
    global argv options installData

    foreach parameter $argv {
        if {![regexp {(?:--)(.*)(?:=)(.*)} $parameter nada option value]} {
            puts "Bad parameter: $parameter"
            ShowUsage
        }
        if {[lsearch [list prefix proxy lang desktop menu personal] $option]==-1} {
            puts "Bad option: $option"
            ShowUsage
        }
        set options($option) $value
    }
    if {![info exists options(prefix)]} {
        set options(prefix) ""
    } else {
        set options(prefix) [string trimright $options(prefix) /]
    }

    set installData(forAll) 1
    if {[info exists options(personal)]} {
        set installData(forAll) $options(personal)
        if {$options(personal)==1} {
            set options(prefix) ""
            set installData(forAll) 0
        } else {
            set installData(forAll) 1
        }
    }
    return
}

################################################################################
# ConsoleInstall
#    If we are running the script from the console.
################################################################################
proc ConsoleInstall {} {
    global argv argc options getleftOptions

    if {$argc==0} {
        ShowUsage
    }
    ProcessArgs
    InstallDirs
    if {([info exists options(lang)])&&(![ValidLang  $options(lang)])} {
        puts "Unsupported language: $options(lang)"
        exit
    }
    if {($options(proxy)!="")&&(![ValidProxy $options(proxy)])} {
        puts "Proxy: $options(proxy) doesn't look right"
        exit
    }

    set getleftOptions(lang) $options(lang)

    return
}

################################################################################
# checkCancel
#     When the user click on cancel or tries to close the window, we will first
#     check that he really means it.
################################################################################
proc CheckCancel {} {

    set what [Dialogos::Dialogo .wizard.question -type yesno -icon question  \
            -message "Are you sure?" -title Exit]

    if {$what!="no"} {
        exit
    }

    focus .wizard
    return
}

################################################################################
# GraphicInstall
################################################################################
proc GraphicInstall {} {
    global getleftState dirGetleft

    if {[catch {package require tile}]} {
        tk_messageBox -type ok -icon error  -title "Missing extension"      \
                -message "You need Tcl's extension tile"
        exit
    }

    if {[info commands ttk::style] ne ""} {
        interp alias "" ttk::setTheme "" ttk::style theme use
        interp alias "" style         "" ttk::style
    }

    if {[catch {ttk::setTheme tileqt}]} {
       style theme use alt
    }

    if {![catch {package require img::png} error]} {
        set getleftState(imgExt) png
    } elseif {![catch {package require tkpng} error]} {
        set getleftState(imgExt) png
    } else {
        set getleftState(imgExt) gif
    }

    if {![info exists dirGetleft(icons)]} {
        set dirGetleft(images)    [file join [pwd] images]
        set dirGetleft(icons)     [file join [pwd] images Crystal]
        set dirGetleft(scripts)   [file join [pwd] scripts]
        set dirGetleft(doc)       [file join [pwd] doc]
        set dirGetleft(languages) [file join [pwd] Languages]
    }

    set ::auto_path [concat $::auto_path [list $dirGetleft(scripts)]]
    source scripts/tkwizard/tkwizard.tcl
    source scripts/Dialogos.tcl
    source scripts/menuEntry.tcl
    source scripts/Ayuda.tcl

    set ::getleftOptions(menus) grey
    Herramientas::ReadSupportedLang

    source scripts/getWizard.tcl

    return
}

if {$tcl_platform(platform)!="unix"} {
    tk_messageBox -title Error -type ok \
        -message "This install is only for Unix systems."
    exit
}

if {[regexp -nocase tclsh [info nameofexecutable]]} {
    set consoleInstall 1
} else {
    set consoleInstall 0
}

set options(proxy) ""
set options(lang)  en

source scripts/Herramientas.tcl
Herramientas::SetGetleftDirs
set ::getleftState(os) unix
set ::wizFull 1

if {$consoleInstall==1} {
    ConsoleInstall
} else {
    GraphicInstall
}

if {$consoleInstall==1} {
    if {$options(prefix) ne ""} {
        set pathCode [ValidPath $options(prefix)]
        if {$pathCode==1} {
            puts "Invalid path: $options(prefix)"
            exit
        } elseif {$pathCode==2} {
            puts "You don't have the rights to install this in '$options(prefix)'"
            exit
        }
    }
    if {([info exists options(desktop)])&&($options(desktop)==1)} {
        Herramientas::CreateShortcut desktop
    }
    if {([info exists options(menu)])&&($options(menu)==1)} {
        Herramientas::CreateShortcut menu
    }
}

if {$installData(forAll)==1} {
    if {$options(prefix)!=""} {
        InstallFiles
    }
    WriteConfFile
} else {
    Herramientas::SaveConfig
}

if {$consoleInstall==1} {
    puts "Install finished"
}

exit

